/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package p4uclient;

/**
 * アプリ全体を管理
 * @author tondol
 */
public class MainApp {
    static ClientManager client = null;
    static TaskListener  listen = null;
    static FileManager   files  = null;
    static OptionManager option = null;
    static TaskManager   tasks  = null;
    static MainView      window = null;
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        window = MainView.getInstance();
        tasks  = new TaskManager();
        option = new OptionManager("config.ini");
    }

    /**
     * 終了時の処理
     */
    public static void exit() {
        try {
            window.setBusy(true);
            window.applyState();
            offline();
            tasks.close();
            option.close();
        } catch(Exception e) {
            e.printStackTrace();
        } finally {
            System.exit(0);
        }
    }
    
    /**
     * 各セッションを開始
     */
    public static void online() {
        try {
            window.setBusy(true);
            window.applyState();
            client = new ClientManager();
            listen = new TaskListener();
            files  = new FileManager("upload.ini");
            window.setOnline(true);
            window.putMessage("o サーバー接続 <" + option.host + ">");
        } catch(Exception e) {
            try {
                if (client != null) client.close();
                if (listen != null) listen.close();
            } catch(Exception ex) {
            } finally {
                window.putMessage("x サーバー接続 <" + option.host + ">");
                window.putMessage("debug: " + e);
            }
        } finally {
            window.setBusy(false);
            window.applyState();
            update();
        }
    }
    
    /**
     * 各セッションを終了
     */
    public static void offline() {
        try {
            window.setBusy(true);
            window.setOnline(false);
            window.applyState();
            if (files  != null) files.close();
            if (client != null) client.close();
            if (listen != null) listen.close();
            window.modelSearch.clearItem();
        } catch(Exception e) {
            window.putMessage("x サーバー切断 <" + client.host + ">");
            window.putMessage("debug: " + e);
        } finally {
            window.setBusy(false);
            window.applyState();
            window.putMessage("o サーバー切断 <" + client.host + ">");
        }
    }
    
    /**
     * 共有リストを更新する操作
     */
    public static void update() {
        try {
            window.setScan(true);
            window.applyState();
            files.updateItems();
            window.putMessage("o リスト更新 <" + option.updir + ">");
        } catch(Exception e) {
            window.putMessage("x リスト更新 <" + option.updir + ">");
            window.putMessage("debug: " + e);
        } finally {
            window.setScan(false);
            window.applyState();
        }
    }
    
    /**
     * サーバーの共有リストを検索
     */
    public static void search(String word) {
        try {
            if (word.length() == 0) word = ".";
            client.search(word);
            window.putMessage("o ファイル検索 <" + word + ">");
        } catch(Exception e) {
            window.putMessage("x ファイル検索 <" + word + ">");
            window.putMessage("debug: " + e);
        }
    }
    
    /**
     * ファイルを受信
     */
    public static void download(FileItem item) {
        try {
            TaskDownload task = new TaskDownload(item);
            tasks.addItem(task);
            window.putMessage("o 受信起動 <" + item.name + ">");
        } catch(Exception e) {
            window.putMessage("x 受信起動 <" + item.name + ">");
            window.putMessage("debug: " + e);
        }
    }
    
    /**
     * タスクをキャンセル
     */
    public static void cancel(TaskAbstract task) {
        try {
            tasks.deleteItem(task);
            window.putMessage("o タスク中断 <" + task.getName() + ">");
        } catch(Exception e) {
            window.putMessage("x タスク中断 <" + task.getName() + ">");
            window.putMessage("debug: " + e);
        }
    }
}
